#!/usr/bin/env python

import sys
import os
import commands
import re
import locale
import pwd
import grp
import gettext
import string
import random
import crypt
import signal
import getopt

##
## I18N and GTK
##

domain = 'oranavi'
gettext.bindtextdomain(domain, '/usr/share/locale')
gettext.textdomain (domain)
_ = gettext.gettext

try:
    display_env = os.environ['DISPLAY']
except KeyError:
    sys.exit(_('Please specify $DISPLAY variable!'))

import gtk
import gtk.glade

gtk.glade.bindtextdomain(domain, '/usr/share/locale')

##
## Icon for windows
##
iconPixbuf = None
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/pixmaps/oranavi/oranavi.png')
except:
    pass

version = 'oranavi-10.2'
fname = '/usr/share/oranavi/oranavi.glade'
pfile = '/usr/share/oranavi/oraparam.ini'

ORANAVI_MENU = 0
ORANAVI_SRC  = 1
ORANAVI_GROUP = 2
ORANAVI_ATR = 3
ORANAVI_ENV1 = 4
ORANAVI_LANG1 = 5
ORANAVI_ENVACK = 6
ORANAVI_ENV3 = 7
ORANAVI_APACHE = 8
ORANAVI_INST = 9
ORANAVI_SELECT = 10

INSTALL_DB = 50
INSTALL_AS = 51
SET_AUTO_START = 52

TRUE = 1
FALSE = 0

saltlist = string.ascii_letters + string.digits + '/.'

force_option = FALSE

db_path  = ['stage/Components/oracle.rdbms',
            'stage/Components/oracle.client',
            'stage/Components/oracle.companionCD.db',
            'stage/Components/oracle.crs']

as_path  = ['stage/Components/oracle.iappserver.iapptop']

db_version = ['9.2.0.1.0', '9.2.0.4.0', '10.1.0.2.0']

as_version = ['9.0.4.0.0']

def disp_err_dialog(self, str):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
			       type=gtk.MESSAGE_ERROR,
			       buttons=gtk.BUTTONS_OK,
			       message_format=str)
    dialog.set_position(gtk.WIN_POS_CENTER)
    ret = dialog.run()
    dialog.destroy()
    return ret

def disp_verify_dialog(self, str):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
			       type=gtk.MESSAGE_INFO,
			       buttons=gtk.BUTTONS_OK_CANCEL,
			       message_format=str)
    dialog.set_position(gtk.WIN_POS_CENTER)
    ret = dialog.run()
    dialog.destroy()
    if ret == gtk.RESPONSE_OK:
	return TRUE
    else:
	return FALSE


def usage():
    print "Usage: oranavi [-h|--help] [-f|--force]"


class OranaviAbst:

    def on_delete_event(self, window, event=None):
	if disp_verify_dialog(self, _('Would you like to quit oranavi?')):
	    gtk.main_quit()

    def on_BtnNext_clicked(self, window):
	self.win.hide()
	self.parent.go_next(self.current_id, self.next_id)

    def on_BtnPrev_clicked(self, window):
	self.win.hide()
	self.parent.go_back()

    def hide(self):
	self.win.hide()

    def show(self):
	self.win.show()

    def is_valid_path(self, name):
	return re.match(r'^/[-a-zA-Z0-9_/.]*$', name)

    def __init__(self, parent, widget, cur, nex):
	self.parent = parent
	self.current_id = cur
	self.next_id = nex
	self.xml = gtk.glade.XML(fname, widget, domain)
	self.win = self.xml.get_widget(widget)
	if iconPixbuf != None:
	    self.win.set_icon(iconPixbuf)
	self.xml.signal_autoconnect(self)


class OranaviMenu(OranaviAbst):

    def on_BtnNext_clicked(self, window):
	if self.action == INSTALL_DB or self.action == INSTALL_AS:
	    self.next_id = ORANAVI_SRC
	    OranaviAbst.on_BtnNext_clicked(self, window)
	else:
	    if not os.path.isfile('/etc/oratab'):
		msg = _('/etc/oratab not found.\nPlease install Oracle first')
		disp_err_dialog(self, msg)
		return 
	    self.next_id = ORANAVI_SELECT
	    OranaviAbst.on_BtnNext_clicked(self, window)
	    self.parent.notify_auto_start()

    def on_rbInstall_toggled(self, window):
	self.action = INSTALL_DB
	self.parent.set_inst_type(INSTALL_DB)

    def on_rbinstIAS_toggled(self, window):
	self.action = INSTALL_AS
	self.parent.set_inst_type(INSTALL_AS)

    def on_rbAuto_toggled(self, window):
	self.action = SET_AUTO_START

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgMenu', ORANAVI_MENU, ORANAVI_SRC)
	self.action = INSTALL_DB
	self.radio = self.xml.get_widget('rbinstIAS')
	self.radio.set_sensitive(FALSE)


class OranaviSrc(OranaviAbst):

    def checks_for_src(self):

	if self.src == 0:
	    src_found = FALSE
	    if os.system('/bin/grep -q /media/cdrom /etc/fstab') == 0 and os.path.isdir('/media/cdrom'):
		if os.system('/bin/grep -q /media/cdrom /proc/mounts') == 0:
		    src_found = TRUE
		    self.entrypath = '/media/cdrom'
		elif os.system('/bin/mount /media/cdrom') == 0:
		    src_found = TRUE
		    self.entrypath = '/media/cdrom'
	    if (not src_found) and os.system('/bin/grep -q /media/cdrecorder /etc/fstab') == 0 and \
			os.path.isdir('/media/cdrecorder'):
		if os.system('/bin/grep -q /media/cdrecorder /proc/mounts') == 0:
		    src_found = TRUE
		    self.entrypath = '/media/cdrecorder'
		elif os.system('/bin/mount /media/cdrecorder') == 0:
		    src_found = TRUE
		    self.entrypath = '/media/cdrecorder'
	    if (not src_found):
		msg = _('Failed to mount Install CD.\nPlease insert it and press OK button.')
		disp_err_dialog(self, msg)
		return FALSE
        else:
	    self.entrypath = self.entrybox.get_text()
	    if self.entrypath == '' or not self.is_valid_path(self.entrypath):
		disp_err_dialog(self, _('Please specify the valid path'))
		return FALSE

	# checks for a trial CD-ROM
	if os.path.isdir(os.path.join(self.entrypath, 'server')):
	    self.entrypath = os.path.join(self.entrypath, 'server')

	inst_type = self.parent.get_inst_type()
	if inst_type == INSTALL_DB:
	    # For a DVD-ROM media, append db or database to its path
	    if os.path.isdir(os.path.join(self.entrypath, 'db')):
		self.entrypath = os.path.join(self.entrypath, 'db')
            elif os.path.isdir(os.path.join(self.entrypath, 'database')):
                self.entrypath = os.path.join(self.entrypath, 'database')
	    if not self.retrieve_version(db_path):
		disp_err_dialog(self, _('It is not a valid Oracle DB source'))
		return FALSE
	    if not force_option and (self.version[0:4] != '10.2' and self.version[0:4] != '10.1' and self.version[0:3] != '9.2'):
		disp_err_dialog(self, _('This version of Oracle is unsupported in Asianux.\nPlease check for the latest release of oranavi.'))
		return FALSE
	    return TRUE
	elif inst_type == INSTALL_AS:
	    # For a DVD-ROM media, append application_server to its path
	    if os.path.isdir(os.path.join(self.entrypath, 'application_server')):
		self.entrypath = os.path.join(self.entrypath, 'application_server')
	    if not self.retrieve_version(as_path):
		disp_err_dialog(self, _('It is not a valid Oracle AS source'))
		return FALSE
	    return TRUE
	return FALSE

    def retrieve_version(self, list):
	found = FALSE
	regexp = re.compile('\d+\.\d+\.\d+')
	for ent in list:
	    current_dir = os.path.join(self.entrypath, ent)
	    if os.path.isdir(current_dir):
		file_list = os.listdir(current_dir)
		if len(file_list) == 1 and regexp.search(file_list[0]):
		    found = TRUE
		    self.version = file_list[0]
		    break
	return found

    def on_BtnNext_clicked(self, window):
	if self.checks_for_src():
	    msg1 = _('The version of Oracle you are installing is ')
	    msg2 = _('\nWould you like to proceed?')
	    if disp_verify_dialog(self, msg1 + self.version + msg2):
		self.parent.set_path(self.entrypath)
		self.parent.set_version(self.version)
		OranaviAbst.on_BtnNext_clicked(self, window)
	    else:
		disp_err_dialog(self, _('Failed to retrieve the version of Oracle.'))

    def on_rbCDROM_toggled(self, window):
	self.src = 0
	self.entrybox.set_editable(FALSE)

    def on_rbHDD_toggled(self, window):
	self.src = 1
	self.entrybox.set_editable(TRUE)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgSrcMedia', ORANAVI_SRC, ORANAVI_GROUP)
	self.src = 0
	self.entrypath = ''
	self.version = ''
	self.entrybox = self.xml.get_widget('etrHDDPath')
	self.entrypath = '/media/cdrom'


class OranaviGroup(OranaviAbst):

    def is_valid_name(self, name):
	return re.match(r'^[a-z][-a-zA-Z0-9_]*$', name)

    def checks_for_valid_input(self):
	self.username = self.userbox.get_text()
	if self.username == '' or not self.is_valid_name(self.username):
	    disp_err_dialog(self, _('Please insert a valid user name.'))
	    return FALSE
	self.instg = self.instgbox.get_text()
	self.adming = self.admingbox.get_text()
	if self.instg == '' or not self.is_valid_name(self.instg) or \
		self.adming == '' or not self.is_valid_name(self.adming):
	    disp_err_dialog(self, _('Please insert valid group names.'))
	    return FALSE
	return TRUE

    def checks_for_user(self):
	try:
	    pwd.getpwnam(self.username)
	    found = TRUE
	except KeyError:
	    found = FALSE
        return found

    def on_BtnNext_clicked(self, window):
	exmsg = _('User specified already exist.\nWould you like to use the existing user?')
	if not self.checks_for_valid_input():
	    return
	if self.checks_for_user():
	    if disp_verify_dialog(self, exmsg):
		self.next_id = ORANAVI_ENV1
	    else:
		return
	else:
	    self.next_id = ORANAVI_ATR
	self.parent.set_user(self.username)
	self.parent.set_instg(self.instg)
	self.parent.set_adming(self.adming)
	OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgGroup', ORANAVI_GROUP, ORANAVI_ENV1)
	self.userbox = self.xml.get_widget('etrUser')
	self.instgbox = self.xml.get_widget('etrGrInstall')
	self.admingbox = self.xml.get_widget('etrGrAdmin')


class OranaviAtr(OranaviAbst):

    def verify_passwd(self):
	self.passph = self.passbox.get_text()
	vrfyph = self.vrfybox.get_text()
	if self.passph == '':
	    disp_err_dialog(self, _('Please specify a passwd.'))
	    return FALSE
	elif self.passph != vrfyph:
	    disp_err_dialog(self, _('Pass phrases do not match.'))
	    return FALSE
	return TRUE

    def verify_homepath(self):
	self.homepath = self.homebox.get_text()
	if self.homepath == '' or not self.is_valid_path(self.homepath):
	    disp_err_dialog(self, _('Please specify a valid home directory.'))
	    return FALSE
	return TRUE

    def notify_new_name(self, name):
	self.homebox.set_text(os.path.join('/home', name))

    def on_BtnNext_clicked(self, window):
	if self.verify_passwd() and self.verify_homepath():
	    self.parent.set_passwd(self.passph)
	    self.parent.set_homepath(self.homepath)
	    OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgAtr', ORANAVI_ATR, ORANAVI_ENV1)
	self.passbox = self.xml.get_widget('etrPasswd')
	self.vrfybox = self.xml.get_widget('etrVerify')
	self.homebox = self.xml.get_widget('etrHdir')
	self.homebox.set_text('/home/oracle')


class OranaviEnv1(OranaviAbst):

    def determine_oraparam(self, version):
	if version[0] == '9':
	    orabase = '/opt/oracle'
	    if version[0:5] == '9.0.4':
		orabase = '/opt/app/oracle'
	    self.orabase.set_text(orabase)
	    self.orahome.set_text(os.path.join(orabase, version))
	elif version[0:2] == '10':
	    self.orabase.set_text('/opt/app/oracle')
	    self.orahome.set_text(os.path.join('/opt/app/oracle/product', version,
				  'db_1'))
	else:
	    self.orabase.set_text('/opt/app/oracle')
	    self.orahome.set_text(os.path.join('/opt/app/oracle/product', version))
	self.orasid.set_text('orcl')

    def notify_version(self, version):
	self.determine_oraparam(version)

    def get_obase(self):
	return self.baseval

    def register_params(self):
	self.baseval = self.orabase.get_text()
	homeval = self.orahome.get_text()
	sidval = self.orasid.get_text()
	if self.baseval == '' or homeval == '' or sidval == '':
	    disp_err_dialog(self, _('Please specify all the values.'))
	    return FALSE
	if not self.is_valid_path(self.baseval) or not self.is_valid_path(homeval):
	    disp_err_dialog(self, _('Please specify valid entries.'))
	    return FALSE
	self.args = '\n# added by oranavi\n' + \
			'export ORACLE_BASE=' + self.baseval + '\n' + \
			'export ORACLE_HOME=' + homeval + '\n' + \
			'export ORACLE_SID=' + sidval + '\n'
	return TRUE

    def on_BtnNext_clicked(self, window):
	if self.register_params():
	    self.parent.set_orahomes(self.args)
	else:
	    return
	if self.check.get_active():
	    self.next_id = ORANAVI_APACHE
	    self.parent.construct_auto_profile()
	else:
	    self.next_id = ORANAVI_LANG1
	OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgEnv1', ORANAVI_ENV1, ORANAVI_APACHE)
	self.orabase = self.xml.get_widget('etrEnv_ORACLE_BASE')
	self.orahome = self.xml.get_widget('etrEnv_ORACLE_HOME')
	self.orasid  = self.xml.get_widget('etrEnv_ORACLE_SID')
	self.check   = self.xml.get_widget('checkEnv')
	self.determine_oraparam(self.parent.get_version())
	self.args = ''


class OranaviLang(OranaviAbst):

    def activate_wrt_encoding(self):
	lang = self.parent.get_lang()
	enc = self.parent.get_encoding()
	if lang == 'ja_JP':
	    if enc == 'SJIS':
		butt = self.xml.get_widget('rbLangSJIS')
		self.nls_lang = 'Japanese_Japan.JA16SJIS'
	    elif enc == 'eucJP':
		butt = self.xml.get_widget('rbLangEUC')
		self.nls_lang = 'Japanese_Japan.JA16EUC'
	    else:
		butt = self.xml.get_widget('rbLangJaUTF8')
		self.nls_lang = 'Japanese_Japan.AL32UTF8'
	elif lang == 'zh_CN':
	    butt = self.xml.get_widget('rbLangSimplified')
	    self.nls_lang = '\"Simplified Chinese_CHINA.ZHS16GBK\"'
	elif lang == 'zh_TW':
	    butt = self.xml.get_widget('rbLangTraditional')
	    self.nls_lang = '\"Traditional Chinese_TAIWAN.ZHT16BIG5\"'
	elif lang == 'ko_KR':
	    butt = self.xml.get_widget('rbLangKorea')
	    self.nls_lang = 'American_America.KO16KSC5601'
 	else:
	    butt = self.xml.get_widget('rbLangEnglish')
	    self.nls_lang = 'American_America.US7ASCII'
	butt.set_active(TRUE)

    def on_rbLangJaUTF8_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'Japanese_Japan.AL32UTF8'
	self.specify_enc = FALSE

    def on_rbLangJapaneseEUC_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'Japanese_Japan.JA16EUC'
	self.specify_enc = FALSE

    def on_rbLangJapaneseSJIS_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'Japanese_Japan.JA16SJIS'
	self.specify_enc = FALSE

    def on_rbLangSimplified_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = '\"Simplified Chinese_CHINA.ZHS16GBK\"'
	self.specify_enc = FALSE

    def on_rbLangTraditional_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = '\"Traditional Chinese_TAIWAN.ZHT16BIG5\"'
	self.specify_enc = FALSE

    def on_rbLangGBK_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'American_America.ZHS16GBK'
	self.specify_enc = FALSE

    def on_rbLangKorea_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'American_America.KO16KSC5601'
	self.specify_enc = FALSE

    def on_rbLangEnglish_toggled(self, window):
	self.editpane.set_editable(FALSE)
	self.nls_lang = 'American_America.US7ASCII'
	self.specify_enc = FALSE

    def on_rbLangOther_toggled(self, window):
	self.editpane.set_editable(TRUE)
	self.specify_enc = TRUE

    def get_def_nls_lang(self):
	return 'export NLS_LANG=' + self.def_nls + '\n'

    def on_BtnNext_clicked(self, window):
	if self.specify_enc:
	    self.nls_lang = self.editpane.get_text()
	    if self.nls_lang == '':
		disp_err_dialog(self, _('Please specify a valid NLS_LANG parameter'))
	        return
	self.parent.set_nls_lang('export NLS_LANG=' + self.nls_lang + '\n')
	OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
        OranaviAbst.__init__(self, parent, 'dlgLang', ORANAVI_LANG1, ORANAVI_ENVACK)
	self.editpane = self.xml.get_widget('etrLangOther')
	self.specify_enc = FALSE
	self.nls_lang = ''
	self.activate_wrt_encoding()
	self.def_nls = self.nls_lang


class OranaviEnvack(OranaviAbst):

    def env_initialise(self):
	envlist = self.xml.get_widget('label52')
	self.envstr =   'export ORA_NLS33=$ORACLE_HOME/ocommon/nls/admin/data\n' + \
			'export PATH=$PATH:$ORACLE_HOME/bin\n' + \
			'export ORACLE_DOC=$ORACLE_HOME/doc\n' + \
			'CLASSPATH=$ORACLE_HOME/jre:$ORACLE_HOME/JRE\n' + \
			'CLASSPATH=$CLASSPATH:$ORACLE_HOME/jlib\n' + \
			'CLASSPATH=$CLASSPATH:$ORACLE_HOME/rdbms/jlib\n' + \
			'CLASSPATH=$CLASSPATH:$ORACLE_HOME/jdbc/lib/classes12.jar\n' + \
			'CLASSPATH=$CLASSPATH:$ORACLE_HOME/jdbc/lib/nls_charset12.jar\n' + \
			'export CLASSPATH\n' + \
			'LD_LIBRARY_PATH=$ORACLE_HOME/lib:$ORACLE_HOME/ctx/lib\n' + \
			'LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$ORACLE_HOME/jdbc/lib\n' + \
			'export LD_LIBRARY_PATH\n'
	envlist.set_text(self.envstr)

    def on_EnvackRbYes_toggled(self, window):
	self.auto = TRUE

    def on_EnvackRbNo_toggled(self, window):
	self.auto = FALSE

    def get_def_auto_var(self):
	return self.envstr

    def on_BtnNext_clicked(self, window):
	if (self.auto):
	    self.parent.set_default_var(self.envstr)
	else:
	    self.parent.set_default_var('')
	OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgEnvack', ORANAVI_ENVACK, ORANAVI_ENV3)
	self.env_initialise()
	self.auto = TRUE


class OranaviEnv3(OranaviAbst):

    def retrieve_extra(self):
	for i in ['1', '2', '3', '4', '5']:
	    name_widget = self.xml.get_widget('etrEnv3Name' + i)
	    var_name = name_widget.get_text()
	    if var_name != '':
		val_widget = self.xml.get_widget('etrEnv3Val' + i)
		var_val = val_widget.get_text()
		if var_val != '':
		    self.extrastr = self.extrastr + 'export ' + var_name + "=" + var_val + "\n"

    def set_val_wrt_version(self):
	lang = self.parent.get_lang()
	version = self.parent.get_version()
	if lang == 'zh_CN' or lang == 'zh_TW' or (lang == 'ko_KR' and version[0:4] != '10.2'):
	    self.extrastr = self.extrastr + 'unset LANG\n'
	inst_type = self.parent.get_inst_type()
	version = self.parent.get_version()
	if version[0] == '9' or version[0:2] == '10':
		self.extrastr = self.extrastr + \
				'ulimit -n 65536 > /dev/null 2>&1\n' + \
				'ulimit -u 16384 > /dev/null 2>&1\n'
	if inst_type == INSTALL_DB:
	    if version[0] == '9':
		self.extrastr = self.extrastr + \
				'export LD_ASSUME_KERNEL=2.4.19\n'
	else:
	    if version[0:7] == '1.0.2.0' or version[0:7] == '1.0.2.1':
		self.extrastr = self.extrastr + \
				'export LD_ASSUME_KERNEL=2.2.5\n'

    def get_def_extended_var(self):
	self.extrastr = ''
	self.set_val_wrt_version()
	return self.extrastr

    def on_BtnNext_clicked(self, window):
	self.extrastr = ''
	self.retrieve_extra()
	self.set_val_wrt_version()
	self.parent.set_extra_var(self.extrastr)
	dlg = OranaviEnvDlg(self.parent.construct_manual_profile())
	if dlg.run():
	    OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgEnv3', ORANAVI_ENV3, ORANAVI_APACHE)


class OranaviEnvDlg:

    def run(self):
	ret = self.dlg.run()
	self.dlg.destroy()
	if ret == gtk.RESPONSE_OK:
	    return TRUE
	else:
	    return FALSE

    def __init__(self, str):
	xml = gtk.glade.XML(fname, 'dlgEnvFinal', domain=domain)
	self.dlg = xml.get_widget('dlgEnvFinal')
	self.dlg.set_position(gtk.WIN_POS_CENTER)
	label = xml.get_widget('envack')
	label.set_text(str)


class OranaviApache(OranaviAbst):

    def write_setting(self):
	if not os.path.isfile('/etc/httpd/conf/httpd.conf'):
	    disp_err_dialog(self, _('Apache is not installed in the system.'))
	    return FALSE
	return TRUE

    def on_ApacheRbYes_toggled(self, window):
	self.write = TRUE

    def on_ApacheRbNo_toggled(self, window):
	self.write = FALSE

    def on_BtnNext_clicked(self, window):
	if self.write and self.write_setting():
	    self.parent.set_httpd_daemon(TRUE)
	else:
	    self.parent.set_httpd_daemon(FALSE)
	OranaviAbst.on_BtnNext_clicked(self, window)

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgApache', ORANAVI_APACHE, ORANAVI_INST)
	self.write = TRUE


class OranaviInstall(OranaviAbst):

    def on_BtnNext_clicked(self, window):
	self.win.hide()
	self.dummy.show()
	self.parent.start_actual_work()
	gtk.main_quit()

    def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgInstall', ORANAVI_INST, ORANAVI_INST)
	dummyxml = gtk.glade.XML(fname, 'dlgDummy')
	self.dummy = dummyxml.get_widget('dlgDummy')


class OranaviSelect(OranaviAbst):

   def modify_dbstart(self, orahome, orauser):
	if os.system('egrep -q \'^[^#].*PFILE=\' ' + orahome + '/bin/dbstart') == 0:
	    os.system('su - ' + orauser + ' -c \'cp ' + orahome + '/bin/dbstart ' + \
		      orahome + '/bin/dbstart.oranavi\'; ' + \
		      'sed \'s/^[^#].*PFILE=/#           PFILE=/\' ' + \
		      orahome + '/bin/dbstart.oranavi > ' + orahome + '/bin/dbstart')

   def modify_dbshut(self, orahome, orauser):
	if os.system('egrep -q \'shutdown[^ ]*$\' ' + orahome + '/bin/dbshut') == 0:
	    os.system('su - ' + orauser + ' -c \'cp ' + orahome + '/bin/dbshut ' + \
		      orahome + '/bin/dbshut.oranavi\'; ' + \
		      'sed \'s/^shutdown[^ ]*$/shutdown immediate/\' ' + \
		      orahome + '/bin/dbshut.oranavi > ' + orahome + '/bin/dbshut')

   def get_oracle_version(self, orahome, orauser):
	if os.path.isfile(orahome + '/bin/sqlplus'):
	    f = os.popen('su - ' + orauser + ' -c \"' + orahome + \
			 '/bin/sqlplus -v | awk \'/^SQL/ ' + \
			 '{print substr(\\$3,1,5)}\'\"')
	    ver = string.strip(str(f.readline()), '\n')
	    f.close()
	    return ver
        else:
	    return None

   def get_oracle_owner(self, orahome):
	if os.path.isfile(orahome + '/bin/dbstart'):
	    f = os.popen('ls -l ' + orahome + '/bin/dbstart | awk \'{print $3}\'', 'r')
	    str = string.strip(f.readline(), '\n')
	    f.close()
	    return str
	else:
	    return None

   def lookup_valid_sid(self):
	self.liststore.clear()
	if not os.path.isfile('/etc/rc.d/init.d/dbora'):
	    disp_err_dialog(self, _('/etc/rc.d/init.d/dbora does not exist.'))
	    f.close()
	    return
	activate = FALSE
	f = os.popen('awk -F: \'/^[^#:]+:.+:[NY]$/ {print $1; print $2; print $3}\' /etc/oratab', 'r')
	current_str = string.strip(f.readline(), '\n')
	orahome_str = string.strip(f.readline(), '\n')
	status_str = string.strip(f.readline(), '\n')
	if current_str == '' or orahome_str == '' or status_str == '':
	    disp_err_dialog(self, _('No entry found in /etc/oratab'))
	    f.close()
	    return
	while current_str != '' and orahome_str != '' and status_str != '':
	    if status_str == 'Y':
		activate = TRUE
	    user = self.get_oracle_owner(orahome_str)
	    version = self.get_oracle_version(orahome_str, user)
	    if user != None and version != None:
		if version[0:1] == '9':
		    self.modify_dbstart(orahome_str, user)
		    self.modify_dbshut(orahome_str, user)
	    self.liststore.append([current_str, status_str])
	    current_str = string.strip(f.readline(), '\n')
	    orahome_str = string.strip(f.readline(), '\n')
	    status_str = string.strip(f.readline(), '\n')
	f.close()
	if activate:
	    os.system('/sbin/chkconfig dbora on')
	else:
	    os.system('/sbin/chkconfig dbora off')

   def on_SelectBtnApply_clicked(self, window):
	if not os.path.isfile('/etc/oratab'):
	    disp_err_dialog(self, _('/etc/oratab not found.'))
	    return
	iter = self.selection.get_selected()[1]
	if iter == None:
	    disp_err_dialog(self, _('Please choose a row'))
	    return
	sid = self.liststore.get_value(iter, 0)
	command = 'sed -n \'s/^' + sid + ':.*:\\([YN]\\)/\\1/p\' /etc/oratab'
	f = os.popen(command, 'r')
	c = string.strip(f.read(), '\n')
	f.close()
	if c == 'Y':
	    nc = 'N'
	else:
	    nc = 'Y'
	pid = str(os.getpid())
	replace = 'sed \'s/\\(^' + sid + ':.*:\\)\\([YN]\\)/\\1' + nc + \
		  '/\' /etc/oratab > /tmp/.oranavi_' + pid + \
		  '; cp /tmp/.oranavi_' + pid + ' /etc/oratab; ' + \
		  'rm -f /tmp/.oranavi_' + pid
	os.system(replace)
	iter = self.logbuffer.get_end_iter()
	self.logbuffer.insert(iter, sid + _(': status changed : ') + \
			c + ' -> ' + nc + '\n')
	self.lookup_valid_sid()

   def __init__(self, parent):
	OranaviAbst.__init__(self, parent, 'dlgSelect', ORANAVI_SELECT, ORANAVI_SELECT)
	self.clist = self.xml.get_widget('SelectClist')
	self.selection = self.clist.get_selection()
	self.selection.set_mode(gtk.SELECTION_SINGLE)
	self.liststore = gtk.ListStore(str, str)
	self.clist.set_model(self.liststore)
	renderer = gtk.CellRendererText()
	sidnam = gtk.TreeViewColumn('SID', renderer, text=0)
	statnam = gtk.TreeViewColumn('Status', renderer, text=1)
	self.clist.append_column(sidnam)
	self.clist.append_column(statnam)
	logbox = self.xml.get_widget('SelectLog')
	self.logbuffer = logbox.get_buffer()


class OranaviMain:

    def get_inst_type(self):
	return self.inst_type

    def set_inst_type(self, num):
	self.inst_type = num

    def notify_auto_start(self):
	self.child[ORANAVI_SELECT].lookup_valid_sid()

    def set_path(self, path):
	self.path = path

    def set_version(self, version):
	self.version = version
	if self.inst_type == INSTALL_DB:
	    if version[0:2] == '10':
		self.version = version[0:6]
	    elif version[0:1] == '9':
		self.version = version[0:5]
	self.child[ORANAVI_ENV1].notify_version(self.version)

    def get_version(self):
	return self.version

    def set_user(self, user):
	self.user = user
	self.child[ORANAVI_ATR].notify_new_name(user)

    def set_instg(self, gname):
	self.instg = gname

    def set_adming(self, gname):
	self.adming = gname

    def set_passwd(self, passwd):
	self.passwd = passwd

    def set_homepath(self, homepath):
	self.homepath = homepath

    def set_orahomes(self, args):
	self.orahomes = args

    def set_nls_lang(self, args):
	self.nlslang = args

    def set_default_var(self, args):
	self.defvar = args

    def set_extra_var(self, args):
	self.extravar = args

    def construct_auto_profile(self):
	self.bashval =  self.orahomes + \
			self.child[ORANAVI_LANG1].get_def_nls_lang() + \
			self.child[ORANAVI_ENVACK].get_def_auto_var() + \
			self.child[ORANAVI_ENV3].get_def_extended_var()

    def construct_manual_profile(self):
	self.bashval = self.orahomes + self.nlslang + self.defvar + self.extravar
	return self.bashval

    def set_httpd_daemon(self, bool):
	self.httpd_set = bool

    def get_lang(self):
	return self.lang

    def get_encoding(self):
	return self.encoding

    def go_back(self):
	self.current = self.child[self.back.pop()]
	self.current.show()

    def go_next(self, current, next):
	self.back.append(current)
	self.current = self.child[next]
	self.current.show()

    def checks_for_group(self, gname):
	try:
	    grp.getgrnam(gname)
	    found = TRUE
	except KeyError:
	    found = FALSE
	return found

    def add_user_and_group(self):
	msg = _('Failed to add a group ')
	if not self.checks_for_group(self.instg):
	   if os.system('/usr/sbin/groupadd ' + self.instg) != 0:
		disp_err_dialog(self, msg + self.instg)
		return FALSE
	if not self.checks_for_group(self.adming):
	   if os.system('/usr/sbin/groupadd ' + self.adming) != 0:
		disp_err_dialog(self, msg + self.adming)
		return FALSE
	if self.passwd != '':
	    crypted = crypt.crypt(self.passwd, random.choice(saltlist) + random.choice(saltlist))
	    if os.system('/usr/sbin/useradd -d ' + self.homepath + \
			 ' -g ' + self.instg + ' -G ' + self.adming + \
			 ' -p ' + crypted + ' ' + self.user) != 0:
		msg = _('Failed to add a user ')
		disp_err_dialog(self, msg + self.user)
                return FALSE
	else:
	    try:
		self.homepath = pwd.getpwnam(self.user)[5]
	    except KeyError:
		disp_err_dialog(self, _('Its a bug!'))
		return FALSE
	    if os.system('/usr/sbin/usermod -g ' + self.instg + ' -G ' + \
			 self.adming + ' ' + self.user) != 0:
		disp_err_dialog(self, _('Failed to join in groups'))
		return FALSE
	return TRUE

    def write_to_files(self):
	msg = _(' does not exist.')
	profile_file = os.path.join(self.homepath, '.bash_profile')
	if not os.path.isfile(profile_file):
	    disp_err_dialog(self, profile_file + msg)
	    return FALSE
	file = os.open(profile_file, os.O_WRONLY|os.O_APPEND)
	os.write(file, self.bashval)
	os.close(file)
	rc_file = os.path.join(self.homepath, '.bashrc')
	if not os.path.isfile(rc_file):
	    disp_err_dialog(self, rc_file + msg)
	if os.system('/bin/grep -q \'^# added by oranavi\' ' + rc_file) != 0:
	    new_ent =   '\n# added by oranavi\n' + \
			'if [ $ORACLE_HOME ] && [ ! $LD_LIBRARY_PATH ]; then\n' + \
			'    LD_LIBRARY_PATH=$ORACLE_HOME/lib:$ORACLE_HOME/ctx/lib:$ORACLE_HOME/jdbc/lib\n' + \
			'    export LD_LIBRARY_PATH\n' + \
			'fi\n'
	    file = os.open(rc_file, os.O_WRONLY|os.O_APPEND)
	    os.write(file, new_ent)
	    os.close(file)
	if self.httpd_set:
	    httpd_file = '/etc/sysconfig/httpd'
	    file = os.open(httpd_file, os.O_CREAT|os.O_WRONLY|os.O_APPEND)
	    os.write(file, self.bashval)
	    os.close(file)
	return TRUE

    ##  Architecture detection
    def getArch (self):
        arch = os.uname ()[4]
        if (len (arch) == 4 and arch[0] == 'i' and
            arch[2:4] == "86"):
             arch = "i386"
        if arch == "sparc64":
            arch = "sparc"
        if arch == "ppc64":
            arch = "ppc"
        if arch == "s390x":
            arch = "s390"
        return arch
  
    def launch_oui(self):
	systemArch=self.getArch()
	print "The system architecture is " + systemArch
	if self.version[0:5] == '9.0.4':
	    installer = os.path.join(self.path, 'install/linux/runInstaller')
	else:
	    installer = os.path.join(self.path, 'runInstaller')
	if not os.path.isfile(installer):
	    disp_err_dialog(self, installer + _(' does not exist.'))
	    return FALSE
	(stat, f) = commands.getstatusoutput('su - ' + self.user + ' -c \'' + \
		    '[ -x ' + installer + ' ] && echo 1 || echo 0 \'')
	if f == str(FALSE):
	    self.current.hide()
	    disp_err_dialog(self, _('Oracle user hasn\'t got sufficient privilege to execute installer.'))
	    return FALSE
	os.system('/usr/X11R6/bin/xhost +')
	command_line = 'su - ' + self.user + ' --command=\"umask 022;unset TNS_ADMIN;'
	if self.lang == 'zh_CN' or self.lang == 'zh_TW' or (self.lang == 'ko_KR' and self.version[0:4] != '10.2'):
	    command_line = command_line + 'unset LANG;'
	if self.version[0:5] == '9.0.4':
	    command_line = command_line + \
			   'unset CLASSPATH;' + \
			   'export LD_ASSUME_KERNEL=2.4.19;' + \
			   'export TMP=/tmp;' + \
			   'DISPLAY=' + display_env + ' ' + installer + ' -noClusterEnabled' + \
			   '\"'
	elif self.version[0:4] == '10.2':
	   if systemArch == 'ia64': ## Checking Itanium2
	       command_line = command_line + \
			      'DISPLAY=' + display_env + ' ' + installer + ' -ignoreSysPrereqs' + \
			      '\"'
	   else:	
	       command_line = command_line + \
	                      'DISPLAY=' + display_env + ' ' + installer + \
		   	      '\"'
	elif self.version[0:2] == '10':
	    if systemArch == 'ia64': ## Checking Itanium2
		command_line = command_line + \
			     'DISPLAY=' + display_env + ' ' + installer + ' -ignoreSysPrereqs' + \
 		             '\"'
	    else:
		command_line = command_line + \
		   	     'DISPLAY=' + display_env + ' ' + installer + ' -paramFile ' + pfile + \
 		   	     '\"'
		
	elif self.version[0:1] == '9':
	    command_line = command_line + \
			   'unset JAVA_HOME;' + \
			   'DISPLAY=' + display_env + ' ' + installer + \
			   '\"'
	os.system(command_line)

    def update_kernel_params(self):
	if self.version[0:4] == '10.2':
	    sedc = ''
	    echoc = ''
	    (stat, f) = commands.getstatusoutput('/bin/cat /proc/sys/net/core/rmem_max | xargs expr \'1048576\' \'<=\' ')
	    if f == '0':
		sedc = sedc + '-e \'s/^net.core.rmem_max/#net.core.rmem_max/\' '
		echoc = echoc + 'net.core.rmem_max=1048576\n'
	    (stat, f) = commands.getstatusoutput('/bin/cat /proc/sys/net/core/rmem_default | xargs expr \'1048576\' \'<=\' ')
	    if f == '0':
		sedc = sedc + '-e \'s/^net.core.rmem_default/#net.core.rmem_default/\' '
		echoc = echoc + 'net.core.rmem_default=1048576\n'
	    (stat, f) = commands.getstatusoutput('/bin/cat /proc/sys/net/core/wmem_max | xargs expr \'262144\' \'<=\' ')
	    if f == '0':
		sedc = sedc + '-e \'s/^net.core.wmem_max/#net.core.wmem_max/\' '
		echoc = echoc + 'net.core.wmem_max=262144\n'
	    (stat, f) = commands.getstatusoutput('/bin/cat /proc/sys/net/core/wmem_default | xargs expr \'262144\' \'<=\' ')
	    if f == '0':
		sedc = sedc + '-e \'s/^net.core.wmem_default/#net.core.wmem_default/\' '
		echoc = echoc + 'net.core.wmem_default=262144\n'
	    if sedc != '' and echoc != '':
		sedc = '/bin/sed ' + sedc + '/etc/sysctl.conf > /etc/.sysctl.conf.tmp; ' + \
		       'mv -f /etc/.sysctl.conf.tmp /etc/sysctl.conf'
		os.system(sedc)
		echoc = '/bin/echo -e \'' + echoc + '\' >> /etc/sysctl.conf'
		os.system(echoc)
		(stat, f) = commands.getstatusoutput('/sbin/sysctl -p')
		return stat
	return TRUE

    def start_actual_work(self):
	if not self.add_user_and_group():
	    gtk.main_quit()
	if not self.write_to_files():
	    gtk.main_quit()
	if not self.update_kernel_params():
	    disp_err_dialog(self, _('Failed to update kernel parameters.'))
	    gtk.main_quit()
	obase = self.child[ORANAVI_ENV1].get_obase()
	os.system('mkdir -p -m775 ' + obase)
	os.system('chgrp ' + self.instg + ' ' + obase + '; chmod 775 ' + obase)
	self.launch_oui()

    def determine_browser(self):
	mozilla = '/usr/bin/mozilla'
	netscape = '/usr/bin/netscape'
	firefox = '/usr/bin/firefox'
	if os.path.isfile(mozilla):
	    browser = mozilla
	elif os.path.isfile(netscape):
	    browser = netscape
	elif os.path.isfile(firefox):
	    browser = firefox
	else:
	    disp_err_dialog(self, _('No valid browser found in the system'))
	    return
	if self.lang == 'ja_JP':
	    lang_dir = 'ja'
	elif self.lang == 'zh_CN':
	    lang_dir = 'zh_CN'
	elif self.lang == 'zh_TW':
	    lang_dir = 'zh_TW'
	elif self.lang == 'ko_KR':
	    lang_dir = 'ko'
	else:
	    lang_dir = 'en'
	index_page = os.path.join('/usr/share/oranavi/doc', lang_dir, 'index.html')
	if not os.path.isfile(index_page):
	    disp_err_dialog(self, _('No doc found for oranavi'))
	    return
	os.spawnl(os.P_NOWAIT, browser, browser, 'file://' + index_page)

    def __init__(self):
	self.inst_type = INSTALL_DB
	self.user = 'oracle'
	self.passwd = ''
	self.version = '10.1.0.2.0'
	self.httpd_set = TRUE
	if os.getuid() != 0 and os.geteuid() != 0:
	    disp_err_dialog(None, _('Oranavi can only be invoked by root user.'))
	    sys.exit()
	(self.lang, self.encoding) = locale.getlocale()
	self.back = []
	self.child = []
	self.child.append(OranaviMenu(self))
	self.child.append(OranaviSrc(self))
	self.child.append(OranaviGroup(self))
	self.child.append(OranaviAtr(self))
	self.child.append(OranaviEnv1(self))
	self.child.append(OranaviLang(self))
	self.child.append(OranaviEnvack(self))
	self.child.append(OranaviEnv3(self))
	self.child.append(OranaviApache(self))
	self.child.append(OranaviInstall(self))
	self.child.append(OranaviSelect(self))
	self.current = self.child[ORANAVI_MENU]
	self.determine_browser()


try:
    opts, args = getopt.getopt(sys.argv[1:], "hf", ["help", "force"])
except getopt.GetoptError:
    usage()
    sys.exit(-1)
for o, a in opts:
    if o in ("-h", "--help"):
	usage()
	sys.exit()
    if o in ("-f", "--force"):
	force_option = TRUE

oranavi = OranaviMain()
signal.signal(signal.SIGINT, gtk.main_quit)
gtk.main()
